/*
 * Copyright (c) 2005 Kuan-li Pong
 *
 * This file is part of the "chess" game library. "chess" is free
 * software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this library; see the file COPYING.  If not, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#ifndef BASERULE_HPP
#define BASERULE_HPP

#include "rule.hpp"

namespace Chess{

	/**
	 * Class BaseRule
	 * 򥻳WhO,WhԨREADME
	 */
	class BaseRule: public Rule{
		private:
			enum InnerState{
				UNKNOWN,
				SELECTING,
				EATING
			};

			const static bool relations[7][7];
		public:
			BaseRule();
			void start(Player& p1, Player& p2);
			void dropGame();
			~BaseRule();

			void clicked(int col, int row, MouseButton button);
		private:
			void initBoard();
			void clearBoard();
			void clearChesses();
			inline void makeOneChess(int id, Chess::Color color);
			inline void makeOnePairChesses(int id);
			void unstarted(int col, int row, MouseButton button);
			void playing(int col, int row, MouseButton button);
			void switchTurn();
			inline Player* getCurrentPlayer();
			inline Player* getTheOtherPlayer();
			inline Turn getTheOtherTurn();
			bool strongerThan(Chess& ch1, Chess& ch2);
			bool canEat(int x1, int y1, int x2, int y2, bool position_only=false);
			bool canReach(int x1, int y1, int x2, int y2);
			bool canEatDefault(int x1, int y1, int x2, int y2);
			bool canPaouEat(int x1, int y1, int x2, int y2);
			bool canReachDefault(int x1, int y1, int x2, int y2);
			inline bool compareColor(Chess::Color color, Turn turn);
		private:
			Chess** chesses;	//]chessboardtdchessO޲z,ҥHbo̺޲z
			int chess_make_count;
			InnerState in_state;
	};
}

#endif
