/*
 * Copyright (c) 2005 Kuan-li Pong
 *
 * This file is part of the "chess" game library. "chess" is free
 * software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this library; see the file COPYING.  If not, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#ifndef CHESSBOARD_CPP
#define CHESSBOARD_CPP

#include "chessboard.hpp"

namespace Chess{

	class Chess;

	template<int N_COLS, int N_ROWS>
	ChessBoard<N_COLS, N_ROWS>::ChessBoard(){
		for(int i = 0; i < N_COLS; ++i)
			for(int j = 0; j < N_ROWS; ++j)
				board[i][j] = 0;
		selected = 0;
	}

	template<int N_COLS, int N_ROWS>
	void ChessBoard<N_COLS, N_ROWS>::setSelectedChess(Chess* chess){
		selected = chess;
	}

	template<int N_COLS, int N_ROWS>
	Chess* ChessBoard<N_COLS, N_ROWS>::getChess(int x, int y){
		if(!examinePosition(x, y))
			return 0;
		return board[x][y];
	}

	template<int N_COLS, int N_ROWS>
	void ChessBoard<N_COLS, N_ROWS>::setChess(int x, int y, Chess* chess){
		if(examinePosition(x, y))
			board[x][y] = chess;
	}

	template<int N_COLS, int N_ROWS>
	void ChessBoard<N_COLS, N_ROWS>::moveChess(int x1, int y1, int x2, int y2){
		if(examinePosition(x1, y1) && examinePosition(x2, y2)){
			board[x2][y2] = board[x1][y1];
			board[x1][y1] = 0;
		}
	}

	template<int N_COLS, int N_ROWS>
	void ChessBoard<N_COLS, N_ROWS>::removeChess(int x, int y){
		if(examinePosition(x, y))
			board[x][y] = 0;
	}

	template<int N_COLS, int N_ROWS>
	void ChessBoard<N_COLS, N_ROWS>::getChessPosition(const Chess* chess, int &x, int &y){
		if(chess){
			for(int col = 0; col < N_COLS; ++col)
				for(int row = 0; row < N_ROWS; ++row)
					if(board[col][row] == chess){
						x = col;
						y = row;
						return;
					}
		}
		x = y = -1;
	}
}

#endif
