/*
 * Copyright (c) 2005 Kuan-li Pong
 *
 * This file is part of the "chess" game library. "chess" is free
 * software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this library; see the file COPYING.  If not, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include "extrule.hpp"
#include "player.hpp"
#include <cstdlib>
#include <iostream>

namespace Chess{

	ExtRule::ExtRule()
		:Rule(){
		chesses = 0;
		std::srand( std::time(NULL) );
	}

	void ExtRule::start(Player& p1, Player& p2){
		Rule::start(p1, p2);
		initBoard();
		turn = PLAYER_1;
		//std::vector<Chess> chs(*chesses, (*chesses)+32);
		std::vector<Chess> chs;
		for(int i = 0; i < 32; i++)
			chs.push_back(*(chesses[i]));
		outputStart(8, 4, chs);
		outputSetPlayerName(PLAYER_1, p1.getName());
		outputSetPlayerName(PLAYER_2, p2.getName());
		outputUpdatePlayer(PLAYER_1, 32);
		outputUpdatePlayer(PLAYER_2, 32);
		outputSetCurrentPlayer(PLAYER_1);
		in_state = UNKNOWN;
	}

	void ExtRule::dropGame(){
		if(status == UNSTARTED || status == PLAYING){
			outputEnd(OutputListener::DROP);
			status = ENDED;
			//۰ʤ_outputsu
			clearOutputListeners();
			clearBoard();
		}
	}

	ExtRule::~ExtRule(){
		//޹CO_i,۰ʤ_outputsu
		outputEnd(OutputListener::DROP);
		clearOutputListeners();
		clearChesses();
		clearBoard();
	}

	void ExtRule::clicked(int col, int row, MouseButton button){
		if(col < 0 || col >= 8 && row < 0 || row >= 4)
			return;

		///
		std::cout << "status=" << status << "  in_state=" << in_state << "     ";
		///
		switch(status){
			case UNSTARTED:
				unstarted(col, row, button);
				break;
			case PLAYING:
				playing(col, row, button);
				break;
			default:
				break;
		}
		///
		std::cout << "status=" << status << "  in_state=" << in_state << std::endl;
		///
	}

	void ExtRule::unstarted(int col, int row, MouseButton button){
		if(button != InputListener::LEFT_BUTTON)
			return;
		Chess* chess = board->getChess(col, row);
		chess->cover();
		outputUncoverChess(col, row);
		if(chess->getColor() == Chess::BLUE){
			player_blue = player1;
			player_red = player2;
		}
		else{
			player_blue = player2;
			player_red = player1;
		}
		outputSetPlayerName(PLAYER_BLUE, player_blue->getName());
		outputSetPlayerName(PLAYER_RED, player_red->getName());
		outputUpdatePlayer(PLAYER_BLUE, player_blue->getChesses());
		outputUpdatePlayer(PLAYER_RED, player_red->getChesses());
		if(chess->getColor() == Chess::BLUE){
			turn = PLAYER_RED;
			outputSetCurrentPlayer(PLAYER_RED);
		}
		else{
			turn = PLAYER_BLUE;
			outputSetCurrentPlayer(PLAYER_BLUE);
		}
		status = PLAYING;
	}

	void ExtRule::playing(int col, int row, MouseButton button){
		Chess* chess = board->getChess(col, row);
		Chess* sel_chess = board->getSelectedChess();
		int sel_col = -1, sel_row = -1;
		if(sel_chess)
			board->getChessPosition(sel_chess, sel_col, sel_row);

		if(button == InputListener::LEFT_BUTTON){
			if(chess == sel_chess)
				return;
			switch(in_state){
				case UNKNOWN:
					if(chess){
						if(chess->isCovered() && compareColor(chess->getColor(), turn)){
							board->setSelectedChess(chess);
							outputSetSelectedChess(col, row, true);
							in_state = SELECTING;
						}
						else if(!chess->isCovered()){
							chess->cover();
							outputUncoverChess(col, row);
							switchTurn();
						}
					}
					break;
				case SELECTING:
					if(chess){
						if(canEat(sel_col, sel_row, col, row)){
							bool covered = chess->isCovered();
							board->removeChess(col, row);
							outputRemoveChess(col, row);
							Player* op = getTheOtherPlayer();
							op->removeChess(chess->getID());
							outputUpdatePlayer(getTheOtherTurn(), op->getChesses());
							board->moveChess(sel_col, sel_row, col, row);
							outputMoveChess(sel_col, sel_row, col, row);
							//Bzӭt
							if(op->getChesses() == 0){
								status = ENDED;
								if(turn == PLAYER_BLUE)
									outputEnd(OutputListener::PLAYER_BLUE_WIN);
								else
									outputEnd(OutputListener::PLAYER_RED_WIN);
							}
							else if(covered){
								in_state = EATING;
							}
							else{
								//tѥuY@u
								switchTurn();
							}
						}
						else if(chess && canEat(sel_col, sel_row, col, row, true) && !chess->isCovered()){
							//QYtѬOY
							chess->cover();
							outputUncoverChess(col, row);
							board->setSelectedChess(0);
							outputSetSelectedChess(sel_col, sel_row, 0);
							switchTurn();
						}
					}
					else{
						//ʴѤl
						if(canReach(sel_col, sel_row, col, row)){
							board->moveChess(sel_col, sel_row, col, row);
							outputMoveChess(sel_col, sel_row, col, row);
							switchTurn();
						}
					}
					break;
				case EATING:
					if(chess && canEat(sel_col, sel_row, col, row)){
						bool covered = chess->isCovered();
						board->removeChess(col, row);
						outputRemoveChess(col, row);
						Player* op = getTheOtherPlayer();
						op->removeChess(chess->getID());
						outputUpdatePlayer(getTheOtherTurn(), op->getChesses());
						board->moveChess(sel_col, sel_row, col, row);
						outputMoveChess(sel_col, sel_row, col, row);
						//Bzӭt
						if(op->getChesses() == 0){
							status = ENDED;
							if(turn == PLAYER_BLUE)
								outputEnd(OutputListener::PLAYER_BLUE_WIN);
							else
								outputEnd(OutputListener::PLAYER_RED_WIN);
						}
						else if(!covered){
							//tѥuY@u
							switchTurn();
						}
					}
					else if(chess && canEat(sel_col, sel_row, col, row, true) && !chess->isCovered()){
						//QYtѬOY
						chess->cover();
						outputUncoverChess(col, row);
						board->setSelectedChess(0);
						outputSetSelectedChess(sel_col, sel_row, 0);
						switchTurn();
					}
					break;
			}
		}
		else{
			switch(in_state){
				case SELECTING:
					//
					board->setSelectedChess(0);
					outputSetSelectedChess(sel_col, sel_row, false);
					in_state = UNKNOWN;
					break;
				case EATING:
					//ܨôU@
					board->setSelectedChess(0);
					outputSetSelectedChess(sel_col, sel_row, false);
					switchTurn();
					break;
			}
		}
	}

	void ExtRule::initBoard(){
		clearBoard();
		clearChesses();
		chesses = new Chess*[32];
		chess_make_count = 0;
		makeOnePairChesses(Chess::CHESS_CHANG);
		makeOnePairChesses(Chess::CHESS_SHI);
		makeOnePairChesses(Chess::CHESS_SHI);
		makeOnePairChesses(Chess::CHESS_SHANG);
		makeOnePairChesses(Chess::CHESS_SHANG);
		makeOnePairChesses(Chess::CHESS_CHU);
		makeOnePairChesses(Chess::CHESS_CHU);
		makeOnePairChesses(Chess::CHESS_MA);
		makeOnePairChesses(Chess::CHESS_MA);
		makeOnePairChesses(Chess::CHESS_PAOU);
		makeOnePairChesses(Chess::CHESS_PAOU);
		makeOnePairChesses(Chess::CHESS_TSU);
		makeOnePairChesses(Chess::CHESS_TSU);
		makeOnePairChesses(Chess::CHESS_TSU);
		makeOnePairChesses(Chess::CHESS_TSU);
		makeOnePairChesses(Chess::CHESS_TSU);
		int t1, t2, t3;
		Chess* chess_tmp;
		for(int i = 0; i < 30; ++i){
			t1 = (std::rand() + std::rand()%19) % 32;
			do{
				t2 = (std::rand() + std::rand()%7) % 32;
				t3 = (std::rand() + std::rand()%13) % 32;
			}while(t2 == t1 || t3 == t2 || t3 == t1);
			chess_tmp = chesses[t1];
			chesses[t1] = chesses[t2];
			chesses[t2] = chesses[t3];
			chesses[t3] = chess_tmp;
		}
		board = new ChessBoard<8,4>();
		for(int i = 0; i < 32; i++)
			board->setChess(i % 8, i / 8, chesses[i]);
	}

	void ExtRule::clearBoard(){
		delete board;
		board = 0;
	}

	void ExtRule::clearChesses(){
		if(chesses){
			for(int i = 0; i < 32; i++)
				delete chesses[i];
			delete [] chesses;
		}
		chesses = 0;
	}

	void ExtRule::makeOneChess(int id, Chess::Color color){
		if(chesses && chess_make_count >= 0 && chess_make_count < 32){
			chesses[chess_make_count++] = new Chess(id, color);
		}
	}

	void ExtRule::makeOnePairChesses(int id){
		makeOneChess(id, Chess::BLUE);
		makeOneChess(id, Chess::RED);
	}

	Player* ExtRule::getCurrentPlayer(){
		Player* p = 0;
		switch(turn){
			case PLAYER_1:
				p = player1;
				break;
			case PLAYER_2:
				p = player2;
				break;
			case PLAYER_BLUE:
				p = player_blue;
				break;
			case PLAYER_RED:
				p = player_red;
				break;
		}
		return p;
	}

	Player* ExtRule::getTheOtherPlayer(){
		Player* p = 0;
		switch(turn){
			case PLAYER_1:
				p = player2;
				break;
			case PLAYER_2:
				p = player1;
				break;
			case PLAYER_BLUE:
				p = player_red;
				break;
			case PLAYER_RED:
				p = player_blue;
				break;
		}
		return p;
	}

	Turn ExtRule::getTheOtherTurn(){
		//Turn t = UNKNOWN;	//Lk]UNKNOWN
		Turn t = PLAYER_1;
		switch(turn){
			case PLAYER_1:
				t = PLAYER_2;
				break;
			case PLAYER_2:
				t = PLAYER_1;
				break;
			case PLAYER_BLUE:
				t = PLAYER_RED;
				break;
			case PLAYER_RED:
				t = PLAYER_BLUE;
				break;
		}
		return t;
	}

	bool ExtRule::strongerThan(Chess& ch1, Chess& ch2){
		return (ch1.getColor() != ch2.getColor() && relations[ch1.getID()][ch2.getID()]);
	}

	bool ExtRule::canEat(int x1, int y1, int x2, int y2, bool position_only){
		Chess& ch1 = *board->getChess(x1, y1);
		Chess& ch2 = *board->getChess(x2, y2);
		if(!position_only && !strongerThan(ch1, ch2))
			return false;

		switch(ch1.getID()){
			case Chess::CHESS_CHANG:
			case Chess::CHESS_SHI:
			case Chess::CHESS_SHANG:
			case Chess::CHESS_MA:
			case Chess::CHESS_TSU:
				return canEatDefault(x1, y1, x2, y2);
			case Chess::CHESS_CHU:
				return canChuEat(x1, y1, x2, y2);
			case Chess::CHESS_PAOU:
				return canPaouEat(x1, y1, x2, y2);
		}

		return false;
	}

	bool ExtRule::canEatDefault(int x1, int y1, int x2, int y2){
		int dx = abs(x1 - x2);
		int dy = abs(y1 - y2);
		if(dx == 0 && dy == 1 || dx == 1 && dy == 0)
			return true;
		return false;
	}

	bool ExtRule::canChuEat(int x1, int y1, int x2, int y2){
		if(x1 == x2){
			int _y1 = std::min(y1, y2);
			int _y2 = std::max(y1, y2);
			int _y;
			for(_y = _y1+1; _y < _y2 && !board->getChess(x1, _y); ++_y)
				;
			if(_y == _y2)
				return true;
		}
		else if(y1 == y2){
			int _x1 = std::min(x1, x2);
			int _x2 = std::max(x1, x2);
			int _x;
			for(_x = _x1+1; _x < _x2 && !board->getChess(_x, y1); ++_x)
				;
			if(_x == _x2)
				return true;
		}
		return false;
	}

	bool ExtRule::canPaouEat(int x1, int y1, int x2, int y2){
		if(x1 == x2){
			int _y1 = std::min(y1, y2);
			int _y2 = std::max(y1, y2);
			int s1, s2;
			for(s1 = _y1+1; s1 < _y2 && !board->getChess(x1, s1); ++s1)
				;
			for(s2 = _y2-1; s2 > _y1 && !board->getChess(x2, s2); --s2)
				;
			if(s1 == s2)
				return true;
		}
		else if(y1 == y2){
			int _x1 = std::min(x1, x2);
			int _x2 = std::max(x1, x2);
			int s1, s2;
			for(s1 = _x1+1; s1 < _x2 && !board->getChess(s1, y1); ++s1)
				;
			for(s2 = _x2-1; s2 > _x1 && !board->getChess(s2, y1); --s2)
				;
			if(s1 == s2)
				return true;
		}
		return false;
	}

	bool ExtRule::canReach(int x1, int y1, int x2, int y2){
		Chess& chess = *board->getChess(x1, y1);

		switch(chess.getID()){
			case Chess::CHESS_CHANG:
			case Chess::CHESS_SHI:
			case Chess::CHESS_SHANG:
			case Chess::CHESS_MA:
			case Chess::CHESS_TSU:
			case Chess::CHESS_PAOU:
				return canReachDefault(x1, y1, x2, y2);
			case Chess::CHESS_CHU:
				return canChuReach(x1, y1, x2, y2);
		}
	}

	bool ExtRule::canReachDefault(int x1, int y1, int x2, int y2){
		int dx = abs(x1 - x2);
		int dy = abs(y1 - y2);

		return (dx == 0 && dy == 1 || dx == 1 && dy == 0);
	}

	bool ExtRule::canChuReach(int x1, int y1, int x2, int y2){
		if(x1 == x2){
			int _y1 = std::min(y1, y2);
			int _y2 = std::max(y1, y2);
			int _y;
			for(_y = _y1+1; _y < _y2 && !board->getChess(x1, _y); ++_y)
				;
			if(_y == _y2)
				return true;
		}
		else if(y1 == y2){
			int _x1 = std::min(x1, x2);
			int _x2 = std::max(x1, x2);
			int _x;
			for(_x = _x1+1; _x < _x2 && !board->getChess(_x, y1); ++_x)
				;
			if(_x == _x2)
				return true;
		}
		return false;
	}

	bool ExtRule::compareColor(Chess::Color color, Turn turn){
		return (color == Chess::BLUE && turn == PLAYER_BLUE || color == Chess::RED && turn == PLAYER_RED);
	}

	void ExtRule::switchTurn(){
		Chess* selected = board->getSelectedChess();
		if(selected){
			int x, y;
			board->getChessPosition(selected, x, y);
			board->setSelectedChess(0);
			outputSetSelectedChess(x, y, false);
		}

		turn = getTheOtherTurn();
		outputSetCurrentPlayer(turn);

		in_state = UNKNOWN;
	}

	const bool ExtRule::relations[7][7] = {
		//N   h     H               ]     
		true,  true,  true,  true,  true,  true,  false,
		false, true,  true,  true,  true,  true,  true,
		false, false, true,  true,  true,  true,  true,
		false, false, false, true,  true,  true,  true,
		false, false, false, false, true,  true,  true,
		true,  true,  true,  true,  true,  true,  true,
		true,  false, false, false, false, false, true
	};
}

