/*
 * Copyright (c) 2005 Kuan-li Pong
 *
 * This file is part of the "chess" game library. "chess" is free
 * software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this library; see the file COPYING.  If not, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include "gtkmm/window.h"
#include "gtkmm/box.h"
#include "gtkmm/statusbar.h"
#include "gtkmm/menubar.h"
#include "gtkmm/menu.h"
#include "gtkmm/menuitem.h"
#include "gtkmm/image.h"
#include "gtkmm/label.h"
#include "gtkmm/drawingarea.h"
#include "gtkmm/entry.h"
#include "gtkmm/dialog.h"

#include "outputlistener.hpp"
#include "inputlistener.hpp"
#include "player.hpp"
#include "chess.hpp"
#include "extrule.hpp"
#include "baserule.hpp"
#include "enums.hpp"
#include <vector>
#include <map>

using namespace Gtk;

namespace Chess{

	class Board;

	/**
	 * Class MainFrame
	 * ϧΤ(D)O,tdCJοX
	 */
	class MainFrame: public Window, OutputListener{
		public:
			MainFrame();
			void onNew();	/* td--}sC */
			void onDrop();	/* td--o */
			void onExit();	/* td--C */
			void onAbout();	/* td--oӹC */

			void setInputListener(InputListener* il);

		public:
			/**
			 * HU~ӦOutputO,tdCX\
			 */
			void start(int n_cols, int n_rows, std::vector<Chess> chesses);
			void end(EndStatus status);
			void setPlayerName(Turn turn, Glib::ustring name);
			void setCurrentPlayer(Turn turn);
			void updatePlayer(Turn turn, int remains);
			void moveChess(int x1, int y1, int x2, int y2);
			void uncoverChess(int x, int y);
			void removeChess(int x, int y);
			void setSelectedChess(int x, int y, bool select);
			void setMessage(Glib::ustring message);

			void setLastRemovedImage(Glib::RefPtr<Gdk::Pixbuf> img);

		private:
			void loadImages();

		private:
			MenuBar *menubar;
			Menu *menu_game, *menu_about;
			Board *board;
			Image *img_player, *img_last_rm;
			Label *lbl_player_blue_title, *lbl_player_blue, *lbl_player_red_title, *lbl_player_red, *lbl_player1_title, *lbl_player1, *lbl_player2_title, *lbl_player2;
			Glib::ustring str_player_blue_pref, str_player_red_pref, str_player1_pref, str_player2_pref;
			Statusbar *status;
			Glib::RefPtr<Gdk::Pixbuf> pxb_player_unknown, pxb_player_blue, pxb_player_red, pxb_player1, pxb_player2;
			int player_blue_remains, player_red_remains, player1_remains, player2_remains;
			Turn turn;

			Rule* local_rule;
	};

	/**
	 * Class Board
	 * ϧΤ,tdCѽLXJ\
	 */
	class Board: public DrawingArea{
		public:
			Board(MainFrame& parent);

		protected:
			bool on_expose_event(GdkEventExpose* e);
			void on_realize();
			bool on_button_press_event(GdkEventButton* e);

		public:
			/**
			 * HU禡POutputListener|,MainFramei@BIsҥ
			 */
			void setInputListener(InputListener* il_);
			void start(int n_cols, int n_rows, std::vector<Chess>& chesses);
			void end(OutputListener::EndStatus status);
			void setCurrentPlayer(Turn turn);
			void moveChess(int x1, int y1, int x2, int y2);
			void uncoverChess(int x, int y);
			void removeChess(int x, int y);
			void setSelectedChess(int x, int y, bool select);

		private:
			void loadImages();
			inline void loadChessImage(std::map<int, Glib::RefPtr<Gdk::Pixbuf> >& imgs_map, int id, std::string img_file);
			void drawChess(int col, int row);
			inline void redraw();
			inline void redraw(int col, int row);

		private:
			MainFrame& parent;
			Glib::RefPtr<Gdk::GC> gc;
			Glib::RefPtr<Gdk::Window> window;
			Gdk::Color color_bg, color_fg, color_select;
			std::vector<Chess> chesses_vec;
			Chess* chesses[8][4];
			std::map<int, Glib::RefPtr<Gdk::Pixbuf> > imgs_blue;
			std::map<int, Glib::RefPtr<Gdk::Pixbuf> > imgs_red;
			Glib::RefPtr<Gdk::Pixbuf> img_uncovered, img_none;
			Chess* selected;
			InputListener* il;
	};

	/**
	 * Class NewGameDialog
	 * ϧΤ(sCܮ)
	 */
	class NewGameDialog: public Dialog{
		public:
			enum RuleSelection{
				NONE,
				BASERULE,
				EXTRULE
			};
		public:
			NewGameDialog();
			const static int ID_CANCEL = 1;
			const static int ID_OK = 2;
			inline RuleSelection getSelectedRule();
			inline Glib::ustring getName1();
			inline Glib::ustring getName2();
		private:
			RadioButton *sel_baserule, *sel_extrule;
			Entry *txt_player1, *txt_player2;
	};
}
