/*
 * Copyright (c) 2005 Kuan-li Pong
 *
 * This file is part of the "chess" game library. "chess" is free
 * software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this library; see the file COPYING.  If not, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include "player.hpp"

namespace Chess{

	Player::Player(Glib::ustring& player_name)
		:name(player_name){
		initChesses();
		state = PLAYING;
	}

	//Player::Player(Player& player)
	//	:Player(player.name) { }

	Player::Player(Player& player)
		:name(player.name){
		initChesses();
		state = PLAYING;
	}

	int Player::getChess(int chess_id){
		switch(chess_id){
			case Chess::CHESS_CHANG:
			case Chess::CHESS_SHI:
			case Chess::CHESS_SHANG:
			case Chess::CHESS_CHU:
			case Chess::CHESS_MA:
			case Chess::CHESS_PAOU:
			case Chess::CHESS_TSU:
				return chesses[chess_id];
		}
		return 0;
	}

	int Player::getChesses(){
		int ret_chesses =
			chesses[Chess::CHESS_CHANG] +
			chesses[Chess::CHESS_SHI] +
			chesses[Chess::CHESS_SHANG] +
			chesses[Chess::CHESS_CHU] +
			chesses[Chess::CHESS_MA] +
			chesses[Chess::CHESS_PAOU] +
			chesses[Chess::CHESS_TSU];
		return ret_chesses;
	}

	void Player::removeChess(int chess_id){
		switch(chess_id){
			case Chess::CHESS_CHANG:
			case Chess::CHESS_SHI:
			case Chess::CHESS_SHANG:
			case Chess::CHESS_CHU:
			case Chess::CHESS_MA:
			case Chess::CHESS_PAOU:
			case Chess::CHESS_TSU:
				if(chesses[chess_id] > 0)
					--chesses[chess_id];
				break;
		}
	}

	void Player::initChesses(){
		chesses[Chess::CHESS_CHANG] = 1;
		chesses[Chess::CHESS_SHI] = 2;
		chesses[Chess::CHESS_SHANG] = 2;
		chesses[Chess::CHESS_CHU] = 2;
		chesses[Chess::CHESS_MA] = 2;
		chesses[Chess::CHESS_PAOU] = 2;
		chesses[Chess::CHESS_TSU] = 5;
	}

	void Player::win(){
		state = WIN;
	}

	void Player::lose(){
		state = LOSE;
	}
}

