/*
 * Copyright (c) 2005 Kuan-li Pong
 *
 * This file is part of the "chess" game library. "chess" is free
 * software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this library; see the file COPYING.  If not, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include "rule.hpp"
#include "chess.hpp"
#include "player.hpp"
#include <algorithm>

namespace Chess{

	Rule::Rule(){
		player1 = player2 = 0;
		player_blue = player_red = 0;
		board = 0;
		turn = UNKNOWN;
		status = UNINITIALIZED;
	}

	//w]|outputlisteneroXq
	//sgp1p2iH쥻player1player2
	void Rule::start(Player& p1, Player& p2){
		Player *old_p1 = player1, *old_p2 = player2;
		player1 = new Player(p1);
		player2 = new Player(p2);
		delete old_p1;
		delete old_p2;
		player_blue = 0;
		player_red = 0;
		turn = UNKNOWN;
		status = UNSTARTED;
	}

	//w]|outputlisteneroXq
	void Rule::dropGame(){
		//clear chesses
		delete board;
		board = 0;
		status = ENDED;
	}

	Rule::~Rule(){
		delete board;
	}

	const Player* Rule::getCurrentPlayer(){
		switch(turn){
			case UNKNOWN:
				return 0;
			case PLAYER_1:
				return player1;
			case PLAYER_2:
				return player2;
			case PLAYER_BLUE:
				return player_blue;
			case PLAYER_RED:
				return player_red;
		}
	}

	Turn Rule::getTurn(){
		return turn;
	}

	void Rule::addOutputListener(OutputListener* ol){
		ols.push_back(ol);
	}

	void Rule::removeOutputListener(OutputListener* ol){
		ols.remove(ol);
	}

	void Rule::clearOutputListeners(){
		ols.clear();
	}

	void Rule::outputStart(int n_cols, int n_rows, std::vector<Chess>& chesses){
		Rule::OutputStart op(n_cols, n_rows, chesses);
		std::for_each(ols.begin(), ols.end(), op);
	}

	void Rule::outputEnd(OutputListener::EndStatus status){
		Rule::OutputEnd op(status);
		std::for_each(ols.begin(), ols.end(), op);
	}

	void Rule::outputSetPlayerName(Turn turn, Glib::ustring name){
		Rule::OutputSetPlayerName op(turn, name);
		std::for_each(ols.begin(), ols.end(), op);
	}

	void Rule::outputUpdatePlayer(Turn turn, int remains){
		Rule::OutputUpdatePlayer op(turn, remains);
		std::for_each(ols.begin(), ols.end(), op);
	}

	void Rule::outputSetCurrentPlayer(Turn turn){
		Rule::OutputSetCurrentPlayer op(turn);
		std::for_each(ols.begin(), ols.end(), op);
	}

	void Rule::outputMoveChess(int x1, int y1, int x2, int y2){
		Rule::OutputMoveChess op(x1, y1, x2, y2);
		std::for_each(ols.begin(), ols.end(), op);
	}

	void Rule::outputUncoverChess(int x, int y){
		Rule::OutputUncoverChess op(x, y);
		std::for_each(ols.begin(), ols.end(), op);
	}

	void Rule::outputRemoveChess(int x, int y){
		Rule::OutputRemoveChess op(x, y);
		std::for_each(ols.begin(), ols.end(), op);
	}

	void Rule::outputSetSelectedChess(int x, int y, bool select){
		Rule::OutputSetSelectedChess op(x, y, select);
		std::for_each(ols.begin(), ols.end(), op);
	}

	void Rule::outputSetMessage(Glib::ustring message){
		Rule::OutputSetMessage op(message);
		std::for_each(ols.begin(), ols.end(), op);
	}
}

